@@ -717,3 +717,6 @@ class CEM(Explainer, FitMixin):
         explanation = Explanation(meta=copy.deepcopy(self.meta), data=data)
 
         return explanation
+
+    def reset_predictor(self, predictor: Union[Callable, tf.keras.Model, 'keras.Model']) -> None:
+        raise NotImplementedError('Resetting a predictor is currently not supported')
