@@ -8,7 +8,7 @@ from alibi.api.defaults import DEFAULT_DATA_INTGRAD, DEFAULT_META_INTGRAD
 from alibi.utils.approximation_methods import approximation_parameters
 from alibi.api.interfaces import Explainer, Explanation
 from tensorflow.keras.models import Model
-from typing import Callable, TYPE_CHECKING, Union, List
+from typing import Callable, TYPE_CHECKING, Union, List, Tuple, Optional
 
 if TYPE_CHECKING:  # pragma: no cover
     import keras  # noqa
@@ -21,7 +21,8 @@ def _compute_convergence_delta(model: Union[tf.keras.models.Model, 'keras.models
                                attributions: List[np.ndarray],
                                start_point: List[np.ndarray],
                                end_point: List[np.ndarray],
-                               target: Union[None, np.ndarray, list]) -> np.ndarray:
+                               target: Optional[List[int]],
+                               has_inputs: bool) -> np.ndarray:
     """
     Computes convergence deltas for each data point. Convergence delta measures how close the sum of all attributions
     is to the difference between the model output at the baseline and the model output at the data point.
@@ -40,32 +41,20 @@ def _compute_convergence_delta(model: Union[tf.keras.models.Model, 'keras.models
         Data points.
     target
         Target for which the gradients are calculated for classification models.
+    has_inputs
+        Whether the model is subclassed (`has_inputs`=False) or not (`has_inputs`=True)
 
     Returns
     -------
         Convergence deltas for each data point.
     """
+    if has_inputs:
+        start_point = [tf.convert_to_tensor(start_point[k], dtype=input_dtypes[k]) for k in range(len(input_dtypes))]
+        end_point = [tf.convert_to_tensor(end_point[k], dtype=input_dtypes[k]) for k in range(len(input_dtypes))]
 
-    if len(start_point) != len(end_point):
-        raise ValueError(f"'start_point' and 'end_point' must have the same length. "
-                         f"'start_point' length: {len(start_point)}. 'end_point length: {len(end_point)}'")
-
-    for i in range(len(attributions)):
-        if end_point[i].shape[0] != attributions[i].shape[0]:
-            raise ValueError(f"`attributions {i}` and `end_point {i}` must match on the first dimension "
-                             f"but found `attributions` first dimension: {attributions[i].shape[0]} "
-                             f"and `end_point` first dimension: {end_point[i].shape[0]}")
-        if start_point[i].shape[0] != attributions[i].shape[0]:
-            raise ValueError(f"`attributions {i}` and `start_point {i}` must match on the first dimension "
-                             f"but found `attributions` first dimension: {attributions[i].shape[0]} "
-                             f"and `start_point` first dimension: {start_point[i].shape[0]}")
-        if start_point[i].shape[0] != end_point[i].shape[0]:
-            raise ValueError(f"`start_point' {i} and `end_point` {i} must match on the first dimension "
-                             f"but found `start_point` first dimension: {start_point[i].shape[0]} "
-                             f"and `end_point` first dimension: {end_point[i].shape[0]}")
-
-    start_point = [tf.convert_to_tensor(start_point[k], dtype=input_dtypes[k]) for k in range(len(input_dtypes))]
-    end_point = [tf.convert_to_tensor(end_point[k], dtype=input_dtypes[k]) for k in range(len(input_dtypes))]
+    else:
+        start_point = tf.convert_to_tensor(start_point)
+        end_point = tf.convert_to_tensor(end_point)
 
     def _sum_rows(inp):
 
@@ -81,7 +70,7 @@ def _compute_convergence_delta(model: Union[tf.keras.models.Model, 'keras.models
     start_out = _run_forward(model, start_point, target)
     end_out = _run_forward(model, end_point, target)
 
-    if (len(model.output_shape) == 1 or model.output_shape[1] == 1) and target is not None:
+    if (len(model.output_shape) == 1 or model.output_shape[-1] == 1) and target is not None:
         target_tensor = tf.cast(target, dtype=start_out.dtype)
         target_tensor = tf.reshape(1 - target_tensor, [len(target), 1])
         sign = 2 * target_tensor - 1
@@ -134,7 +123,7 @@ def _run_forward(model: Union[tf.keras.models.Model, 'keras.models.Model'],
         return ps
 
     preds = model(x)
-    if len(model.output_shape) > 1 and model.output_shape[1] > 1:
+    if len(model.output_shape) > 1 and model.output_shape[-1] > 1:
         preds = _select_target(preds, target)
 
     return preds
@@ -295,7 +284,7 @@ def _format_input_baseline(X: np.ndarray,
 
 
 def _format_target(target: Union[None, int, list, np.ndarray],
-                   nb_samples: int) -> list:
+                   nb_samples: int) -> Union[None, List[int]]:
     """
     Formats target to return a list.
 
@@ -361,7 +350,7 @@ def _calculate_sum_int(batches: List[List[tf.Tensor]],
         shape = tuple(shape.as_list())
 
     # invert sign of gradients for target 0 examples if classifier returns only positive class probability
-    if (len(model.output_shape) == 1 or model.output_shape[1] == 1) and target is not None:
+    if (len(model.output_shape) == 1 or model.output_shape[-1] == 1) and target is not None:
         sign = 2 * target_paths - 1
         grads = np.array([s * g for s, g in zip(sign, grads)])
 
@@ -376,8 +365,7 @@ class IntegratedGradients(Explainer):
 
     def __init__(self,
                  model: Union[tf.keras.Model, 'keras.Model'],
-                 layer: Union[tf.keras.layers.Layer, 'keras.layers.Layer',
-                              List[tf.keras.layers.Layer], List['keras.layers.Layer']] = None,
+                 layer: Union[tf.keras.layers.Layer, 'keras.layers.Layer'] = None,
                  method: str = "gausslegendre",
                  n_steps: int = 50,
                  internal_batch_size: int = 100
@@ -393,8 +381,7 @@ class IntegratedGradients(Explainer):
         model
             Tensorflow or Keras model.
         layer
-            Layers with respect to which the gradients are calculated.
-            It can be a single layer or a list of layers.
+            Layer with respect to which the gradients are calculated.
             If not provided, the gradients are calculated with respect to the input.
         method
             Method for the integral approximation. Methods available:
@@ -409,25 +396,26 @@ class IntegratedGradients(Explainer):
         params = locals()
         remove = ['self', 'model', '__class__', 'layer']
         params = {k: v for k, v in params.items() if k not in remove}
-        if not isinstance(layer, list) and layer is not None:
-            layer = [layer]
-        layer_num: Union[int, List]
+        self.model = model
+
+        # subclassed models which have not been called will not have inputs set, we handle this with a flag
+        if self.model.inputs is None:
+            self._has_inputs = False
+            self.inputs = None
+            self.input_dtypes = None
+        else:
+            self._has_inputs = True
+            self.inputs = self.model.inputs
+            self.input_dtypes = [inp.dtype for inp in self.inputs]
+
         if layer is None:
             layer_num = 0
         else:
-            layer_num = []
-            for lay in layer:
-                layer_num.append(model.layers.index(lay))
+            layer_num = model.layers.index(layer)
+
         params['layer'] = layer_num
         self.meta['params'].update(params)
-        self.model = model
         self.layer = layer
-        if not isinstance(self.model.input, list):
-            self.inputs = [self.model.input]
-        else:
-            self.inputs = self.model.input
-        self.input_dtypes = [inp.dtype for inp in self.inputs]
-
         self.n_steps = n_steps
         self.method = method
         self.internal_batch_size = internal_batch_size
@@ -461,43 +449,138 @@ class IntegratedGradients(Explainer):
             for each feature.
 
         """
+        if not self._has_inputs:
+            if isinstance(X, list):
+                inps = [tf.keras.Input(shape=xx.shape[1:], dtype=xx.dtype) for xx in X]
+                self.model(inps)
+                self.inputs = [inp for inp in inps]
+                self.input_dtypes = [inp.dtype for inp in inps]
+                self._has_inputs = True
+            else:
+                inp = tf.keras.Input(shape=X.shape[1:], dtype=X.dtype)
+                self.model(inp)
+                self.inputs = [inp]
+                self.input_dtypes = [inp.dtype]
 
-        if (len(self.model.output_shape) == 1 or self.model.output_shape[1] == 1) and target is None:
+        if (len(self.model.output_shape) == 1 or self.model.output_shape[-1] == 1) and target is None:
             logger.warning("It looks like you are passing a model with a scalar output and target is set to `None`."
                            "If your model is a regression model this will produce correct attributions. If your model "
                            "is a classification model, targets for each datapoint must be defined. "
                            "Not defining the target may lead to incorrect values for the attributions."
                            "Targets can be either the true classes or the classes predicted by the model.")
 
-        if not isinstance(X, list):
-            X = [X]
-            baselines = [baselines]
-        elif isinstance(X, list) and baselines is None:
-            baselines = [None for _ in range(len(X))]
-        elif isinstance(X, list) and baselines is not None:
-            if not isinstance(baselines, list):
-                raise ValueError(f"If the input X is a list, baseline can only be `None` or "
-                                 f"a list of the same length of X. Found baselines type {type(baselines)}")
-            else:
-                if len(X) != len(baselines):
-                    raise ValueError(f"Length of 'X' must match length of 'baselines'. "
-                                     f"Found len(X): {len(X)}, len(baselines): {len(baselines)}")
-
-        if max([len(x) for x in X]) != min([len(x) for x in X]):
-            raise ValueError("First dimension must be equal for all inputs")
-
-        nb_samples = len(X[0])
+        if isinstance(X, list):
+            nb_samples = len(X[0])
+        elif isinstance(X, np.ndarray):
+            nb_samples = len(X)
+        else:
+            raise ValueError("Input must be a np.ndarray or a list of np.ndarray")
 
         # defining integral method
         step_sizes_func, alphas_func = approximation_parameters(self.method)
         step_sizes, alphas = step_sizes_func(self.n_steps), alphas_func(self.n_steps)
         target = _format_target(target, nb_samples)
 
+        if self._has_inputs:
+            # functional/sequential models or subclassed models with multiple inputs
+            if not isinstance(X, list):
+                X = [X]
+                baselines = [baselines]
+            elif isinstance(X, list) and baselines is None:
+                baselines = [None for _ in range(len(X))]
+            elif isinstance(X, list) and baselines is not None:
+                if not isinstance(baselines, list):
+                    raise ValueError(f"If the input X is a list, baseline can only be `None` or "
+                                     f"a list of the same length of X. Found baselines type {type(baselines)}")
+                else:
+                    if len(X) != len(baselines):
+                        raise ValueError(f"Length of 'X' must match length of 'baselines'. "
+                                         f"Found len(X): {len(X)}, len(baselines): {len(baselines)}")
+
+            if max([len(x) for x in X]) != min([len(x) for x in X]):
+                raise ValueError("First dimension must be egual for all inputs")
+
+            attributions, baselines = self._compute_attributions_list_input(X,
+                                                                            baselines,
+                                                                            target,
+                                                                            step_sizes,
+                                                                            alphas,
+                                                                            nb_samples)
+        else:
+            # subclassed models with a single input
+            attributions, baselines = self._compute_attributions_tensor_input(X,
+                                                                              baselines,
+                                                                              target,
+                                                                              step_sizes,
+                                                                              alphas,
+                                                                              nb_samples)
+
+        return self.build_explanation(
+            X=X,
+            baselines=baselines,
+            target=target,
+            attributions=attributions
+        )
+
+    def build_explanation(self,
+                          X: List[np.ndarray],
+                          baselines: List[np.ndarray],
+                          target: Optional[List[int]],
+                          attributions: List[np.ndarray]) -> Explanation:
+        data = copy.deepcopy(DEFAULT_DATA_INTGRAD)
+        data.update(X=X,
+                    baselines=baselines,
+                    target=target,
+                    attributions=attributions)
+
+        # calculate predictions
+        predictions = self.model(X).numpy()
+        data.update(predictions=predictions)
+
+        # calculate convergence deltas
+        deltas = _compute_convergence_delta(self.model,
+                                            self.input_dtypes,
+                                            attributions,
+                                            baselines,
+                                            X,
+                                            target,
+                                            self._has_inputs)
+        data.update(deltas=deltas)
+
+        return Explanation(meta=copy.deepcopy(self.meta), data=data)
+
+    def _compute_attributions_list_input(self,
+                                         X: List[np.ndarray],
+                                         baselines: Union[List[int], List[float], List[np.ndarray]],
+                                         target: Optional[List[int]],
+                                         step_sizes: List[float],
+                                         alphas: List[float],
+                                         nb_samples: int) -> Tuple:
+        """For each tensor in a list of input tensors,
+        calculates the attributions for each feature or element of layer.
+
+        Parameters
+        ----------
+        X
+            Instance for which integrated gradients attribution are computed.
+        baselines
+            Baselines (starting point of the path integral) for each instance.
+        target
+            Defines which element of the model output is considered to compute the gradients.
+        step_sizes
+            Weights in the path integral sum.
+        alphas
+            Interpolation parameter defining the points of the interal path.
+        nb_samples
+            Total number of samples.
+        Returns
+        -------
+            Tuple with integrated gradients attribution and formatted baselines
+
+        """
         # fix orginal call method for layer
         if self.layer is not None:
-            orig_calls = []
-            for layer in self.layer:
-                orig_calls.append(layer.call)
+            orig_calls = self.layer.call
         else:
             orig_calls = None
 
@@ -551,34 +634,30 @@ class IntegratedGradients(Explainer):
             paths_b = [tf.dtypes.cast(paths_b[i], self.input_dtypes[i]) for i in range(len(paths_b))]
 
             if self.layer is not None:
-                grads_b = []
-                for layer_idx in range(len(self.layer)):
-                    grad_b = _gradients_layer(self.model, self.layer[layer_idx],
-                                              orig_calls[layer_idx], paths_b, target_b)
-                    grads_b.append(grad_b)
+                grads_b = _gradients_layer(self.model, self.layer, orig_calls,
+                                           paths_b, target_b)
             else:
                 grads_b = _gradients_input(self.model, paths_b, target_b)
+
             batches.append(grads_b)
 
-        if self.layer is not None:
-            batches = [[batches[i][j] for i in range(len(batches))] for j in range(len(self.layer))]
-        else:
+        if self.layer is None:
+            # multi-input
             batches = [[batches[i][j] for i in range(len(batches))] for j in range(len(self.inputs))]
 
         # calculate attributions from gradients batches
         attributions = []
         if self.layer is not None:
-            for j in range(len(self.layer)):
-                sum_int = _calculate_sum_int(batches, self.model,
-                                             target, target_paths,
-                                             self.n_steps, nb_samples,
-                                             step_sizes, j)
-                layer_output = self.layer[j].output
-                model_layer = Model(self.model.input, outputs=layer_output)
-                norm = (model_layer(X) - model_layer(baselines)).numpy()
+            sum_int = _calculate_sum_int([batches], self.model,
+                                         target, target_paths,
+                                         self.n_steps, nb_samples,
+                                         step_sizes, 0)
+            layer_output = self.layer.output
+            model_layer = Model(self.model.input, outputs=layer_output)
+            norm = (model_layer(X) - model_layer(baselines)).numpy()
 
-                attribution = norm * sum_int
-                attributions.append(attribution)
+            attribution = norm * sum_int
+            attributions.append(attribution)
         else:
             for j in range(len(self.inputs)):
                 sum_int = _calculate_sum_int(batches, self.model,
@@ -590,30 +669,98 @@ class IntegratedGradients(Explainer):
                 attribution = norm * sum_int
                 attributions.append(attribution)
 
-        return self.build_explanation(
-            X=X,
-            baselines=baselines,
-            target=target,
-            attributions=attributions
-        )
+        return attributions, baselines
 
-    def build_explanation(self,
-                          X: List[np.ndarray],
-                          baselines: List[np.ndarray],
-                          target: list,
-                          attributions: List[np.ndarray]) -> Explanation:
-        data = copy.deepcopy(DEFAULT_DATA_INTGRAD)
-        data.update(X=X,
-                    baselines=baselines,
-                    target=target,
-                    attributions=attributions)
+    def _compute_attributions_tensor_input(self,
+                                           X: np.ndarray,
+                                           baselines: Union[int, float, np.ndarray],
+                                           target: Optional[List[int]],
+                                           step_sizes: List[float],
+                                           alphas: List[float],
+                                           nb_samples: int) -> Tuple:
+        """For a single input tensor, calculates the attributions for each input feature or element of layer.
 
-        # calculate predictions
-        predictions = self.model(X).numpy()
-        data.update(predictions=predictions)
+        Parameters
+        ----------
+        X
+            Instance for which integrated gradients attribution are computed.
+        baselines
+            Baselines (starting point of the path integral) for each instance.
+        target
+            Defines which element of the model output is considered to compute the gradients.
+        step_sizes
+            Weights in the path integral sum.
+        alphas
+            Interpolation parameter defining the points of the interal path.
+        nb_samples
+            Total number of samples.
+        Returns
+        -------
+            Tuple with integrated gradients attribution and formatted baselines
+        """
+        # fix orginal call method for layer
+        if self.layer is not None:
+            orig_calls = self.layer.call
+        else:
+            orig_calls = None
 
-        # calculate convergence deltas
-        deltas = _compute_convergence_delta(self.model, self.input_dtypes, attributions, baselines, X, target)
-        data.update(deltas=deltas)
+        # format and check baselines
+        baselines = _format_input_baseline(X, baselines)
 
-        return Explanation(meta=copy.deepcopy(self.meta), data=data)
+        # construct paths
+        paths = np.concatenate([baselines + alphas[i] * (X - baselines) for i in range(self.n_steps)], axis=0)
+
+        # define target paths
+        if target is not None:
+            target_paths = np.concatenate([target for _ in range(self.n_steps)], axis=0)
+        else:
+            target_paths = None
+
+        if target_paths is not None:
+            paths_ds = tf.data.Dataset.from_tensor_slices((paths, target_paths)).batch(self.internal_batch_size)
+        else:
+            paths_ds = tf.data.Dataset.from_tensor_slices(paths).batch(self.internal_batch_size)
+
+        paths_ds.prefetch(tf.data.experimental.AUTOTUNE)
+
+        # calculate gradients for batches
+        batches = []
+        for path in paths_ds:
+
+            if target is not None:
+                paths_b, target_b = path
+            else:
+                paths_b, target_b = path, None
+
+            if self.layer is not None:
+                grads_b = _gradients_layer(self.model, self.layer, orig_calls,
+                                           paths_b, target_b)
+            else:
+                grads_b = _gradients_input(self.model, paths_b, target_b)
+
+            batches.append(grads_b)
+
+        # calculate attributions from gradients batches
+        attributions = []
+        if self.layer is not None:
+            sum_int = _calculate_sum_int([batches], self.model,
+                                         target, target_paths,
+                                         self.n_steps, nb_samples,
+                                         step_sizes, 0)
+            layer_output = self.layer.output
+            model_layer = Model(self.model.input, outputs=layer_output)
+            norm = (model_layer(X) - model_layer(baselines)).numpy()
+
+            attribution = norm * sum_int
+            attributions.append(attribution)
+        else:
+            sum_int = _calculate_sum_int([batches], self.model,
+                                         target, target_paths,
+                                         self.n_steps, nb_samples,
+                                         step_sizes, 0)
+            norm = X - baselines
+
+            attribution = norm * sum_int
+            attributions.append(attribution)
+
+        return attributions, baselines
