@@ -71,7 +71,7 @@ apidoc_extra_args = ['-d 6']
 
 # mock imports
 autodoc_mock_imports = ['sklearn', 'skimage', 'requests',
-                        'cv2', 'bs4', 'keras', 'seaborn', 'PIL', 'tensorflow', 'spacy', 'catboost']
+                        'cv2', 'keras', 'seaborn', 'PIL', 'tensorflow', 'spacy', 'catboost']
 
 # Napoleon settings
 napoleon_google_docstring = True
